use strict;
use warnings;
use Module::Build;

my $builder = Module::Build->new(
    module_name         => 'Statistics::Zed',
    license             => 'perl',
    dist_author         => 'Roderick Garton <rgarton@cpan.org>',
    dist_version_from   => 'lib/Statistics/Zed.pm',
    build_requires => {
        'Test::More' => 0,
        'Class::OOorNo' => 0.01,
		'Math::Cephes' => 0.44,
		'Statistics::Distributions' => 1.02,
        'Statistics::Descriptive' => 2.6,
    },
    add_to_cleanup      => [ 'Statistics-Zed-0.022-*' ],
);

$builder->create_build_script();
